
#pragma once

// Fix an error, that occurs if a previousli included
// file uses '#define PAGE_SIZE ...' which then is used
// by the pre processor to replace the enum name in line
// 79 of <base/attached_ram_dataspace.h>
#undef PAGE_SIZE

// Genode includes
#include <base/attached_ram_dataspace.h>
#include <base/env.h>

// gapfruit includes
#include <tee/communication_buffer.h>
#include <tee/packet_allocator.h>
#include <tee/session_env.h>
#include <tee_session/rpc_object.h>


namespace Tee {

	using namespace Genode;
	using namespace Tee;

	class Session_resources;
	class Session_component_base;
}


class Tee::Session_resources
{
	protected:

		Session_env&          _session_env;
		Heap                  _alloc        { _session_env, _session_env };
		Tee::Packet_allocator _packet_alloc { &_alloc };
		Communication_buffer  _tx_buf;
		Communication_buffer  _rx_buf;

	public:

		Session_resources(Session_env& session_env, const size_t tx_buf_size, const size_t rx_buf_size) :
			_session_env  { session_env },
			_alloc        { _session_env, _session_env },
			_packet_alloc { &_alloc },
			_tx_buf       { _session_env, tx_buf_size },
			_rx_buf       { _session_env, rx_buf_size }
		{ }

		const Session_env& session_env() const { return _session_env; }
};


class Tee::Session_component_base : private Session_resources,
                                    public Tee::Session_rpc_object
{
	protected:

		Entrypoint&  _ep;

		/**
		 * Sub-classes must implement this function, it is called upon all
		 * packet-stream signals.
		 */
		virtual void _handle_packet_stream() = 0;

		void _dispatch() { _handle_packet_stream(); }

		Signal_handler<Session_component_base> _packet_stream_dispatcher {
			_ep, *this, &Session_component_base::_dispatch };

	public:

		/**
		 * Constructor
		 *
		 * \param session_env        environment root the session
		 * \param tx_buf_size        buffer size for tx channel
		 * \param rx_buf_size        buffer size for rx channel
		 */
		Session_component_base(Session_env&     session_env,
		                       const size_t     tx_buf_size,
		                       const size_t     rx_buf_size)
		:
			Session_resources  { session_env, tx_buf_size, rx_buf_size },
			Session_rpc_object { _session_env, _tx_buf.ds(), _rx_buf.ds(), &_packet_alloc, _session_env.ep().rpc_ep() },
			_ep                { _session_env.env().ep() }
		{
			/* install data-flow signal handlers for both packet streams */
			_tx.sigh_ready_to_ack(_packet_stream_dispatcher);
			_tx.sigh_packet_avail(_packet_stream_dispatcher);
			_rx.sigh_ready_to_submit(_packet_stream_dispatcher);
			_rx.sigh_ack_avail(_packet_stream_dispatcher);
		}

		const Session_env& session_env() const { return Session_resources::session_env(); }
};
